surfaceScalarField rAUf("rAUf", fvc::interpolate(rAU));

volVectorField HbyA("HbyA", U);
HbyA = rAU*UEqn().H();

if (pimple.nCorrPISO() <= 1)
{
    UEqn.clear();
}

            surfaceScalarField phiHbyA 
            (
                "phiHbyA",
                (fvc::interpolate(HbyA) & mesh.Sf())
              + fvc::interpolate(rAU)*fvc::ddtCorr(U, phi)   
            );

fvOptions.makeRelative(phiHbyA);

            adjustPhi(phiHbyA, U, p);                           

// Update the fixedFluxPressure BCs to ensure flux consistency
setSnGrad<fixedFluxPressureFvPatchScalarField>
(
    p.boundaryField(),
    (
        phiHbyA.boundaryField()
      - fvOptions.relative(mesh.Sf().boundaryField() & U.boundaryField())
    )/(mesh.magSf().boundaryField()*rAUf.boundaryField())
);

                // Non-orthogonal pressure corrector loop
while (pimple.correctNonOrthogonal())
                {
                    // Pressure corrector
                    fvScalarMatrix pEqn
                    (
                    fvm::laplacian(rAUf, p) == fvc::div(phiHbyA)     
                    );

                    pEqn.setReference(pRefCell, pRefValue);

    pEqn.solve(mesh.solver(p.select(pimple.finalInnerIter())));
                    
		    phiTemp = phiHbyA - pEqn.flux();

//************** START OF INSERTION FOR OUTLET PRESSURE CALCULATION AND ASSIGNMENT **************


		    if ( Pstream::parRun() )  // parallel run
		    {
		      word outletPatchNameArray[nOutlets];
		      label outletPatchArray[nOutlets];
		      for (int i=0; i<nOutlets; i++) 
		      {
			std::ostringstream convertNum2StrBla;
			convertNum2StrBla << i+1;
			outletPatchNameArray[i] = "OUTLET"+convertNum2StrBla.str();
			outletPatchArray[i] = mesh.boundaryMesh().findPatchID(outletPatchNameArray[i]);
		      }
		      
		      // get outfluxes:
		      for(int outletCount=0; outletCount<nOutlets; outletCount++)
		      {
			scalarField tmpOutflux(nProcessors,0);
			tmpOutflux[thisProcNb] = sum(/*phi*/phiTemp.boundaryField()[outletPatchArray[outletCount]]);
			reduce( tmpOutflux, sumOp<scalarField>() );
			
			// sum outfluxes of all processors:
			outFluxArray[outletCount] = 0;
			for (int co=0; co<nProcessors; co++)
			{ 
			  outFluxArray[outletCount] += tmpOutflux[co];
			}
		      }
 
		      // calculate new p values @ outlets and scatter
		      if (count > 0)
		      {
			for(int outletCount=0;outletCount<nOutlets;outletCount++)
			{
			  newPressureValueArray[outletCount] = 0.01*(capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter // capillaryPressure at computed Time
								+ (1/rho) * resistanceArray[outletCount] * outFluxArray[outletCount]);
/*			  if(newPressureValueArray[outletCount] < (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter) + (inletPressureVector - (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter))*0.9)
			  {
				newPressureValueArray[outletCount] = (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter) + (inletPressureVector - (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter))*0.9;
			  }
                          else if(newPressureValueArray[outletCount] > inletPressureVector*1.05)
                          {
                              newPressureValueArray[outletCount] = inletPressureVector*1.05;
                          }
*/


//			  Info << "(1/rho) * resistanceArray[outletCount] * outFluxArray[outletCount]: " << (1/rho) * resistanceArray[outletCount] * outFluxArray[outletCount] << endl; 
//			  Info << "outFluxArray[outletCount]: " << outFluxArray[outletCount] << endl;
			  scalar tmpNewPressureValue;
			  for (int outletCount=0; outletCount<nOutlets; outletCount++)
			  {
			    tmpNewPressureValue = newPressureValueArray[outletCount];
			    Pstream::scatter(tmpNewPressureValue);  // broadcast new p value to all slave processors
			    newPressureValueArray[outletCount] = tmpNewPressureValue;
			  }
			}
		      }
		      else
		      {
			for(int outletCount=0;outletCount<nOutlets;outletCount++)
			{
			  newPressureValueArray[outletCount] = 0.01*(capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter // capillaryPressure at computed Time
								+ (1/rho) *resistanceArray[outletCount] * initialFluxArray[outletCount]);
//			  Info << "(1/rho) * resistanceArray[outletCount] * initialFluxArray[outletCount]: " << (1/rho) * resistanceArray[outletCount] * initialFluxArray[outletCount] << endl;
/*                          if(newPressureValueArray[outletCount] <  (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter) + (inletPressureVector - (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter))*0.9)
                          {
                                newPressureValueArray[outletCount] = (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter) + (inletPressureVector - (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter))*0.9;
                          }
                          else if(newPressureValueArray[outletCount] > inletPressureVector*1.05)
                          {
                                newPressureValueArray[outletCount] = inletPressureVector*1.05;
                          } 
*/
			  scalar tmpNewPressureValue;
			  for (int outletCount=0; outletCount<nOutlets; outletCount++)
			  {
			    tmpNewPressureValue = newPressureValueArray[outletCount];
			    Pstream::scatter(tmpNewPressureValue);  // broadcast new p value to all slave processors
			    newPressureValueArray[outletCount] = tmpNewPressureValue;
			  }
			}
		      }
		      
		      // set new p values @ outlets
		      for(int outletCount=0;outletCount<nOutlets;outletCount++)
		      {
			if ( isOutletArray[outletCount] > 0 )
			{
			  p.boundaryField()[outletPatchArray[outletCount]] == newPressureValueArray[outletCount];
			}
		      }
		    
		    }
		    
		    
		    
		    
		    else  // serial run (one processor)
		    {	
		      word outletPatchNameArray[nOutlets];
		      label outletPatchArray[nOutlets];
		      for (int i=0; i<nOutlets; i++) 
		      {
			std::ostringstream convertNum2StrBla;
			convertNum2StrBla << i+1;
			outletPatchNameArray[i] = "OUTLET"+convertNum2StrBla.str();
			outletPatchArray[i] = mesh.boundaryMesh().findPatchID(outletPatchNameArray[i]);
		      }
		      
		      // sum outfluxes:
		      for(int outletCount=0; outletCount<nOutlets; outletCount++)
		      {
			outFluxArray[outletCount] = sum(/*phi*/phiTemp.boundaryField()[outletPatchArray[outletCount]]);
		      }
      
		      // calculate new p values @ outlets:
		      if (count > 0)
		      {
			for(int outletCount=0;outletCount<nOutlets;outletCount++)
			{
			  newPressureValueArray[outletCount] = 0.01*(capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter // capillaryPressure at computed Time
								+ (1/rho) * resistanceArray[outletCount] * outFluxArray[outletCount]);
//			  newPressureValueArray[outletCount] = (1/rho) * resistanceArray[outletCount] * outFluxArray[outletCount][cycleCounter][timeCounter];
//			  Info << "(1/rho) * resistanceArray[outletCount] * outFluxArray[outletCount]: " << (1/rho) * resistanceArray[outletCount] * outFluxArray[outletCount] << endl; 
//			  Info << "outFluxArray[outletCount]: " << outFluxArray[outletCount] << endl;
/*			  if(newPressureValueArray[outletCount] <  (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter) + (inletPressureVector - (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter))*0.9)
			  {
				newPressureValueArray[outletCount] = (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter) + (inletPressureVector - (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter))*0.9;
			  }
                          else if(newPressureValueArray[outletCount] > inletPressureVector*1.05)
                          {
                                newPressureValueArray[outletCount] = inletPressureVector*1.05;
                          }
*/
			}
		      }
		      else
		      {
			for(int outletCount=0;outletCount<nOutlets;outletCount++)
			{
			  newPressureValueArray[outletCount] = 0.01*(capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter // capillaryPressure at computed Time
								+ (1/rho) *resistanceArray[outletCount] * initialFluxArray[outletCount]);
//			  Info << "(1/rho) * resistanceArray[outletCount] * initialFluxArray[outletCount]: " << (1/rho) * resistanceArray[outletCount] * initialFluxArray[outletCount] << endl; 
  /*                        if(newPressureValueArray[outletCount] <  (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter) + (inletPressureVector - (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter))*0.9)
                          {
                                newPressureValueArray[outletCount] = (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter) + (inletPressureVector - (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] + (capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter))*0.9;
                          }
                          else if(newPressureValueArray[outletCount] > inletPressureVector*1.05)
                          {
                                newPressureValueArray[outletCount] = inletPressureVector*1.05;
                          } 
*/
			  scalar tmpNewPressureValue;
			  for (int outletCount=0; outletCount<nOutlets; outletCount++)
			  {
			    tmpNewPressureValue = newPressureValueArray[outletCount];
			    Pstream::scatter(tmpNewPressureValue);  // broadcast new p value to all slave processors
			    newPressureValueArray[outletCount] = tmpNewPressureValue;
			  }
			}
		      }
		      
		      // set new p values @ outlets
		      for(int outletCount=0;outletCount<nOutlets;outletCount++)
		      {
			if ( isOutletArray[outletCount] > 0 )
			{
			  p.boundaryField()[outletPatchArray[outletCount]] == newPressureValueArray[outletCount];
			}
		      }
		    }
			    
//************** END OF INSERTION FOR OUTLET PRESSURE CALCULATION AND ASSIGNMENT **************

			    
                    if (pimple.finalNonOrthogonalIter())
                    {
                          phi = phiHbyA - pEqn.flux();   
                    }
                }
                
                
                #include "continuityErrs.H"

// Explicitly relax pressure for momentum corrector
p.relax();

                U = HbyA - rAU*fvc::grad(p);	
                U.correctBoundaryConditions();
		fvOptions.correct(U);
