/*---------------------------------------------------------------------------*\
 *  =========                 |
 *  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
 *   \\    /   O peration     |
 *    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
 *     \\/     M anipulation  |
 * -------------------------------------------------------------------------------
 * License
 *    This file is part of OpenFOAM.
 * 
 *    OpenFOAM is free software: you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 * 
 *    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
 *    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *    for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 *    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Application
 *    icoFoamScalarTransport
 * 
 * Description
 *    ...
 * 
 * \*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "DataEntry.H"
#include "subCycle.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
  #include "setRootCase.H"
  Foam::Time runTime("controlDict2", args);  
  #include "createMesh.H"
  #include "createFields.H"
  #include "initContinuityErrs.H"  
  #include <sstream>
  

  
  // * * * * Read pulse properties * * * * * * * * * * * * * //
  
  Info<< "Reading pulse properties ..." << nl << endl;
  
  IOdictionary controlDictCopy
  ( IOobject("controlDict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE ) );
  
//  autoPtr<DataEntry<scalar> > pulseDuration
//  ( DataEntry<scalar>::New("pulseDuration", controlDictCopy) );  
  
  autoPtr<DataEntry<scalar> > nOutletsValue
  (  DataEntry<scalar>::New ("nOutlets", controlDictCopy) );
  int nOutlets = nOutletsValue().value(0);  
  
  IOdictionary controlDictCopy2
  ( IOobject("controlDict2",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE) );

  autoPtr<DataEntry<scalar> > pulseDuration
  ( DataEntry<scalar>::New("pulseDuration", controlDictCopy2) );  //obtain pulseDuration from controlDict2!!
  
  autoPtr<DataEntry<scalar> > startTime
  ( DataEntry<scalar>::New("startTime",controlDictCopy2) );
  
  autoPtr<DataEntry<scalar> > nSubCyclesValue
  ( DataEntry<scalar>::New("Subcycles",controlDictCopy2) );
  int nSubCycles = nSubCyclesValue().value(0);
  Info << "No. of subcycles: " << nSubCycles << nl;
  
    
  // organize time objects:
  scalar pulseT = pulseDuration().value(0); 
  scalar dt1 = runTime.deltaT().value();
  
  double ratio = pulseT / dt1; 
  // round ratio:	
  double ratioRounded = ratio * 10000.0;
  int tmp = ratioRounded + 0.5;
  float tmpFloat = tmp;
  ratioRounded = tmpFloat / 10000.0;
  int nTimeStepsInPulse1 = ratioRounded;  
  
  scalar breakTime = startTime().value(0);  // time where periodicFoam was interupted = new start time of runTime
  runTime.value() = breakTime;
  
  Info << "nTimeStepsInPulse1: " << nTimeStepsInPulse1 << nl;
  Info << "breakTime: " << breakTime << nl;
    
   
  // create inner-cycle time:
  Foam::Time runTime2("controlDict2", args);   
  
  scalar cycleStartTime = breakTime - (nTimeStepsInPulse1)*dt1;
  runTime2.value() = cycleStartTime;
  
  Info << "cycleStartTime: " << cycleStartTime << nl << endl;
  
      
  word inletPatchName = "INLET"; 
  label inletPatch = mesh.boundaryMesh().findPatchID(inletPatchName);    
  word outletPatchNameArray[nOutlets];
  label outletPatchArray[nOutlets];
  for (int i=0; i<nOutlets; i++) 
  {
    std::ostringstream convertNum2Str;
    convertNum2Str << i+1;
    outletPatchNameArray[i] = "OUTLET"+convertNum2Str.str();
    outletPatchArray[i] = mesh.boundaryMesh().findPatchID(outletPatchNameArray[i]);
    Info << "outletPatchArray:" << outletPatchArray[i] << nl;
  }  
  
  label thisProcNb = Pstream::myProcNo();
  int nProcessors = Pstream::nProcs();
  
  // get inlet patch size and area:

/*  scalarField totalInletSize(nProcessors,0);
  scalar localInletSize = c.boundaryField()[inletPatch].size();
  totalInletSize[thisProcNb] = localInletSize;
  reduce( totalInletSize, sumOp<scalarField>() );    
  scalar totalInletSizeSum = 0;
*/

  scalarField totalInletArea(nProcessors,0);
  scalar localInletArea = sum(mesh.magSf().boundaryField()[inletPatch]);
  totalInletArea[thisProcNb] = localInletArea;
  reduce( totalInletArea, sumOp<scalarField>() );
  scalar totalInletAreaSum = 0;

  for (int co=0; co<nProcessors; co++)
  { 
//	totalInletSizeSum += totalInletSize[co]; 
	totalInletAreaSum += totalInletArea[co];
  }
//  Info << nl << "inlet patch size: " << totalInletSizeSum << nl;
  Info << nl << "inlet patch area: " << totalInletAreaSum << nl;
//  Info << nl << "inlet patch area (no sum over procs): " << localInletArea << nl;

  //get outlet patch sizes and areas:
/*   
  scalar totalOutletSizes[nOutlets];
  scalar tmpSize = 0;
*/
  scalar totalOutletAreas[nOutlets];
  scalar tmpArea = 0;

  for(int outletCount=0; outletCount<nOutlets; outletCount++)
  {
/*
    scalarField tmpOutletSize(nProcessors,0);
    tmpSize = c.boundaryField()[outletPatchArray[outletCount]].size();
    tmpOutletSize[thisProcNb] = tmpSize;
    reduce( tmpOutletSize, sumOp<scalarField>() );
    scalar totalOutletSizeSum = 0;
*/
    scalarField tmpOutletArea(nProcessors,0);
    tmpArea = sum(mesh.magSf().boundaryField()[outletPatchArray[outletCount]]);
    tmpOutletArea[thisProcNb] = tmpArea;
    reduce( tmpOutletArea, sumOp<scalarField>() );
    scalar totalOutletAreaSum = 0;

    for (int co=0; co<nProcessors; co++)
    { 
//      totalOutletSizeSum += tmpOutletSize[co];
      totalOutletAreaSum += tmpOutletArea[co];
    }
//    totalOutletSizes[outletCount] = totalOutletSizeSum;
    totalOutletAreas[outletCount] = totalOutletAreaSum;
//    Info << "outlet" << outletCount+1 << " patch size: " << totalOutletSizes[outletCount] << nl; 
    Info << "outlet" << outletCount+1 << " patch area: " << totalOutletAreas[outletCount] << nl; 
//    Info << "outlet" << outletCount+1 << " patch area (no sum over procs): " << totalOutletAreas[outletCount] << nl; 
  }
    
  // create new mesh:
  Foam::fvMesh mesh2
  ( Foam::IOobject(Foam::fvMesh::defaultRegion,runTime2.timeName(),runTime2,Foam::IOobject::MUST_READ) );
     
    
  Info << endl << "********** Starting CA transport loop **********" << nl << endl;
  

  while (runTime.loop())
  {
    
    Info<< "runTime = " << runTime.timeName() << nl;
        
    runTime2.operator++();    
    
    // * * * * if beginning of new cycle reached: reset cycle-time * * * * * * * * * * * //
    
    scalar t = runTime.value();
    ratio = t / pulseT;
    
    // round ratio:	
    ratioRounded = ratio * 10000000.0;
    tmpFloat = ratioRounded + 0.5;
    int ratioInt = tmpFloat / 10000000.0;
    float modulo = t - static_cast<float> (ratioInt) * pulseT;
    double moduloRounded = modulo * 10000000.0;
    tmpFloat = moduloRounded + 0.5;
    moduloRounded = tmpFloat / 10000000.0;    

    
    Info << "moduloRounded: " << moduloRounded << nl;
    
    Info << "t: " << t  << nl;
    Info << "breakTime: " << breakTime  << nl;
    
    if ( t > breakTime )
    {
      if ( moduloRounded < 0.0000001 )  // new cycle!
      {
	Info << nl << "New cycle started!" << nl << endl;	
	
	// reset cycle-time to startTime:
	runTime2.value() = cycleStartTime - dt1;
	runTime2.operator++();         
      }
    }
   
    Info<< "Reading field phi" << nl;
    surfaceScalarField phiTemp
    ( IOobject("phi",runTime2.timeName(),mesh2,IOobject::READ_IF_PRESENT,IOobject::NO_WRITE ), mesh2 ); 
    
    
    if (nSubCycles > 1)
    {
	for
	(
	    subCycle<volScalarField> cSubCycle(c, nSubCycles);
	    !(++cSubCycle).end();
	)
	{      
	  fvScalarMatrix cEqn
	  (
	    fvm::ddt(c)
	    + fvm::div(phiTemp, c)
	    -fvc::div(D * fvc::grad(c))
	  );      
	  cEqn.solve(); 
	}
    }
    else
    {
	  fvScalarMatrix cEqn
	  (
	    fvm::ddt(c)
	    + fvm::div(phiTemp, c)
	    -fvc::div(D * fvc::grad(c))
	  );      
	  cEqn.solve(); 
    }
          
 /*     
  // get inlet & outlet concentration :
  scalarField totalInletConcentration(nProcessors,0);
  scalar localInletConcentration = sum(c.boundaryField()[inletPatch]);
  totalInletConcentration[thisProcNb] = localInletConcentration;
  reduce( totalInletConcentration, sumOp<scalarField>() );    
  scalar totalInletConcentrationSum = 0;
  for (int co=0; co<nProcessors; co++)
  { totalInletConcentrationSum += totalInletConcentration[co]; }
  Info << "inlet concentration sum:" << totalInletConcentrationSum << nl;
   
  for(int outletCount=0; outletCount<nOutlets; outletCount++)
  {
    scalarField tmpOutletConcentration(nProcessors,0);
    tmpOutletConcentration[thisProcNb] = sum(c.boundaryField()[outletPatchArray[outletCount]]);
    reduce( tmpOutletConcentration, sumOp<scalarField>() );
    scalar totalOutletConcentrationSum = 0;
    for (int co=0; co<nProcessors; co++)
    { 
      totalOutletConcentrationSum += tmpOutletConcentration[co];
    }
      Info << "outlet" << outletCount+1 << " concentration sum:" << totalOutletConcentrationSum << nl;
  }
*/

  // get inlet concentration average:
  scalarField totalInletConcentrationAverage(nProcessors,0);
  scalar localInletConcentrationAverage = sum(
	mesh.magSf().boundaryField()[inletPatch]
	*c.boundaryField()[inletPatch]
	) / totalInletAreaSum;
  totalInletConcentrationAverage[thisProcNb] = localInletConcentrationAverage;
  reduce( totalInletConcentrationAverage, sumOp<scalarField>() );    
  scalar totalInletConcentrationAverageSum = 0;
  for (int co=0; co<nProcessors; co++)
  { totalInletConcentrationAverageSum += totalInletConcentrationAverage[co]; }
  Info << "inlet concentration average:" << totalInletConcentrationAverageSum << nl;

  //get outlet concentration averages:
   
  for(int outletCount=0; outletCount<nOutlets; outletCount++)
  {
    scalarField tmpOutletConcentrationAverage(nProcessors,0);
    tmpOutletConcentrationAverage[thisProcNb] = sum(
	mesh.magSf().boundaryField()[outletPatchArray[outletCount]]
	*c.boundaryField()[outletPatchArray[outletCount]]
	) / totalOutletAreas[outletCount];
    reduce( tmpOutletConcentrationAverage, sumOp<scalarField>() );
    scalar totalOutletConcentrationAverageSum = 0;
    for (int co=0; co<nProcessors; co++)
    { 
      totalOutletConcentrationAverageSum += tmpOutletConcentrationAverage[co];
    }
      Info << "outlet" << outletCount+1 << " concentration average:" << totalOutletConcentrationAverageSum << nl;
  }
           
    
    runTime.write();
    
    Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
    << "  ClockTime = " << runTime.elapsedClockTime() << " s"
    << nl << endl;       
  }  
  return 0;
}
