#include <iostream>
#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>
#include <fstream>
#include <sstream>
#include <stdint.h>
#include <string>

using namespace std;

double changingCapacitorVoltage(double timeIncrement, double previousVoltage, double current, double previousCurrent, double capacity, double density)
{
	double result=0;
	result = previousVoltage + 1/capacity * (current - previousCurrent)/2 *timeIncrement/density;
	return result;
}

double chargingCondensator(double timeIncrement, double externalVoltage, double previousVoltage, double timeConstant)
{
	double result = 0;
	result = (externalVoltage - previousVoltage)*(1 - exp(-timeIncrement/timeConstant)) + previousVoltage;
//	cout << "Charging condensator value: " << result << endl;
	return result;
}

double dischargingCondensator(double timeIncrement, double externalVoltage, double previousVoltage, double timeConstant)
{
	double result = 0;
	result = (previousVoltage - externalVoltage)*(exp(-timeIncrement/timeConstant)) + externalVoltage;
//	cout << "Discharging condensator value: " << result << endl;
	return result;
}

int getNumberOfLines(string file) // function to obtain number of Lines from File with myocardial Pressure. Name of file is inserted in command line when programm is executed.
{
	int number_of_lines=0;
	string line;
	ifstream myfile(file.c_str());

	if(myfile.is_open())
	{
		while(!myfile.eof())
		{
			{
				getline(myfile,line);
				if (line.empty()) // make sure, only non-blank lines are counted!
				{
//					cout << "Empty line." << endl;
				}
				else
				{
//					cout << line << endl;
					number_of_lines++;
				}
			}
		}
		myfile.close();
	}

//	cout << "Number of lines in " << file << " is " << number_of_lines << endl;
	
	return number_of_lines;
}

int main()
{
	bool stressRestState;
	double R_arterial=345.6e8, R_capillary1=72e8, R_capillary2=72e8, R_venous=86.4e8;
	double capacitance=0;
	cout << "if rest state enter 0, if stress state enter 1: " ;
	cin >> stressRestState;
	if(stressRestState == 0)
	{
		cout << " REST STATE! " << endl;
		capacitance = 2.75e-11;
		cout << "total arterial resistance R_arterial: " << R_arterial << " kg/(m^4*s)." << endl
		     << "capillary Resistance 1: R_capillary1: " << R_capillary1 << " kg/(m^4*s)." << endl
		     << "capillary Resistance 2: R_capillary2: " << R_capillary2 << " kg/(m^4*s)." << endl
		     << "venous Resistance: R_venous: " << R_venous << " kg/(m^4*s)." << endl
		     << "compliance of intramyocardial vessels: capacitance: " << capacitance << " m^4*s^2/kg." << endl;
	}
	else if(stressRestState == 1)
	{
		cout << " STRESS STATE! " << endl;
		capacitance = 6.75e-11;
		R_arterial *= 0.25;//estimated total resistance of one artery in model and arterioles behind
//		R_capillary1 = 72e8;  //capillary resistance does not change under stress!
//		R_capillary2 = 72e8;
		R_venous *=0.02;
		cout << "total arterial resistance R_arterial: " << R_arterial << " kg/(m^4*s) (reduced by 86%)." << endl
		     << "capillary Resistance 1: R_capillary1: " << R_capillary1 << " kg/(m^4*s) (unchanged)." << endl
		     << "capillary Resistance 2: R_capillary2: " << R_capillary2 << " kg/(m^4*s) (unchanged)." << endl
		     << "venous Resistance: R_venous:" << R_venous << " kg/(m^4*s) (reduced by 98%)." << endl
		     << "compliance of intramyocardial vessels: capacitance: " << capacitance << " m^4*s^2/kg." << endl;
	}

	double R_star=0, R_normal=0;
	R_star = (R_arterial + R_capillary1)*(R_capillary2 + R_venous)/(R_arterial + R_capillary1 + R_capillary2 + R_venous);
	R_normal = R_capillary2 + R_venous;
	
	cout << "Effective resistance for dis/-charging across both branches of the circuit: " << R_star << " kg/(m^4*s)." << endl;	
	cout << "Effective resistance for dis/-charging across only venous branch of the circuit: " << R_normal << " kg/(m^4*s)." <<endl;		

	double capacitorTimeConstant_star = 0, capacitorTimeConstant_normal = 0;
	capacitorTimeConstant_star = capacitance * R_star;
	capacitorTimeConstant_normal = capacitance * R_normal;

	cout << "TimeConstant of capacitor for dis/-charging across both branches of the circuit: " << capacitorTimeConstant_star << " s" << endl;	
	cout << "TimeConstant of capacitor for dis/-charging across only venous branch of the circuit: " << capacitorTimeConstant_normal << " s" << endl;	
	

	int numberOfTimesteps;
	string filename;
	// choose input file with list of outlet radii (in m)
	cout << "What is the name of the input-File with ventricular pressure timesteps for one cycle (list WITHOUT times!!)?" << endl;
	cin >> filename;
	
	string filename1;
	// choose input file with list of outlet radii (in m)
	cout << "What is the name of the input-File with inlet pressure timesteps for one cycle (list WITHOUT times!!)?" << endl;
	cin >> filename1;
	
	numberOfTimesteps = getNumberOfLines(filename);
	cout << "Number of timesteps in myocardial pressure file is: " << numberOfTimesteps << endl;

	double cycleDuration = 1;
	cout << "Insert cardiac cycle duration (in s): " << endl;	
	cin >> cycleDuration;
	double timeStepSize=0;
	timeStepSize = cycleDuration/numberOfTimesteps;
	cout << "time step size is: " << timeStepSize << " s" << endl;	


	double myocardialPressureArray[numberOfTimesteps];
// readin of pressure timesteps from file into myocardialPressureArray
	ifstream infile(filename.c_str());
	int s=0;
	string line;
	stringstream Str;

	if(infile.is_open())
	{
		while(!infile.eof())
		{
			{
				getline(infile,line);
				if (line.empty()) //make sure empty lines are not read into array.
				{
//					cout << "Empty line." << endl;
				}
				else
				{
					Str << line;
					Str >> myocardialPressureArray[s];
					Str.clear(); // Stringstream needs to be cleared in order to be written correctly in next step.
//					cout << "outletRadius" << s << ":" << outletRadiusArray[s] << endl;
					s++;
				}
			}
		}
		infile.close();
	}
	
	double inletPressureArray[numberOfTimesteps];		
	
// readin of inlet pressure timesteps from file into inletPressureArray
	ifstream infile1(filename1.c_str());
	int t=0;
	string line1;
	stringstream Str1;

	if(infile1.is_open())
	{
		while(!infile1.eof())
		{
			{
				getline(infile1,line1);
				if (line1.empty()) //make sure empty lines are not read into array.
				{
//					cout << "Empty line." << endl;
				}
				else
				{
					Str1 << line1;
					Str1 >> inletPressureArray[t];
					Str1.clear(); // Stringstream needs to be cleared in order to be written correctly in next step.
//					cout << "inletPressureArray" << s << ": " << inletPressureArray[t] << endl;
					t++;
				}
			}
		}
		infile1.close();
	}	

	double rho=1060;
	cout << " density rho: " << rho << " kg/m^3. " << endl;
	
		// choose tolerance for termination criterion
	double tolerance = 0.005;
	cout << "termination criterion for capacitor pressure is " << tolerance << " m^2/s^2, corresponding to ~" << tolerance*rho << " kg/(m*s^2) or ~" << tolerance*rho/133.322 << " mmHg . " << endl;

	
//	double newCapacitorVoltage[2*numberOfTimesteps];
//	int maxNumberOfIterations=0;
	int maxNumberOfCycles=50;
	cout << "maximum number of cycles to compute before automatic termination: " << maxNumberOfCycles << "." << endl;
//	maxNumberOfIterations = maxNumberOfCycles*numberOfTimesteps;
	double capacitorVoltage[maxNumberOfCycles][numberOfTimesteps];

	cout << "Enter initial capacitor voltage in m^2/s^2: " << endl;
	cin >> capacitorVoltage[0][numberOfTimesteps-1];


	double resistanceRatio = (R_capillary2 + R_venous)/(R_arterial + R_capillary1 + R_capillary2 + R_venous);
	cout << "ratio of resistance in venous branch to total resistance: " << resistanceRatio << endl;

	double meanRightAtrialPressure = 0;
	cout << "Enter mean right atrial pressure in m^2/s^2: (3.75 mmHg ~ 500 kg/(m*s^2) ~ 0.47 m^2/s^2" << endl;
	cin >> meanRightAtrialPressure;
	cout << "meanRightAtrialPressure: " << meanRightAtrialPressure << " m^2/s^2." << endl;		



	double modelCurrent[numberOfTimesteps], venousCurrent[numberOfTimesteps], capacitorCurrent[numberOfTimesteps]; //required for computation of new capacitorVoltage. all currents are positive in direction of the mutual node of the three branches. Like this all three currents should add to zero!

	modelCurrent[numberOfTimesteps-1] = rho*(inletPressureArray[numberOfTimesteps-1] - (myocardialPressureArray[numberOfTimesteps-1] + capacitorVoltage[0][numberOfTimesteps-1]))/(R_arterial + R_capillary1);
	venousCurrent[numberOfTimesteps-1] = rho*(meanRightAtrialPressure - (myocardialPressureArray[numberOfTimesteps-1] + capacitorVoltage[0][numberOfTimesteps-1]))/(R_capillary2 + R_venous);
	capacitorCurrent[numberOfTimesteps-1] = - modelCurrent[numberOfTimesteps-1] - venousCurrent[numberOfTimesteps-1];

	cout << "initial modelCurrent (volume flow): " << modelCurrent[numberOfTimesteps-1] << " m^3/s." << endl;	
	cout << "initial venousCurrent (volume flow): " << venousCurrent[numberOfTimesteps-1] << " m^3/s." << endl;	
	cout << "initial capacitorCurrent (volume flow): " << capacitorCurrent[numberOfTimesteps-1] << " m^3/s." << endl;			

	// choose name of output file with list table of times and capacitorVoltage
//	string outputfilename;
//	cout << "What is the desired outputfilename?" << endl;
//	cin >> outputfilename;
//	ofstream finalOutput(outputfilename.c_str());

	ofstream output;
	ofstream output2;
	output.open ("control.txt");


// for loop for different capillaryPressureTimestepFiles:

	double tmpMyocardialPressureArray[numberOfTimesteps];
	double myocardialPressureScalingFactor=0;
	for(int K=0; K<7; K++)
	{
		cout << "myocardialPressureScalingFactor: " << 0.65+K*0.1 << ". " << endl;
		std::stringstream numberedFilenames2;
		std::stringstream numberedFilenames;
		myocardialPressureScalingFactor=0.65+K*0.1;
		numberedFilenames << "capillaryPressureTimesteps" << myocardialPressureScalingFactor*100 << ".txt";
		numberedFilenames2 << "capacitorPressureTimesteps" << myocardialPressureScalingFactor*100 << ".txt";
		ofstream finalOutput(numberedFilenames.str().c_str());
		ofstream finalOutput2(numberedFilenames2.str().c_str());
		for(int L=0; L<numberOfTimesteps; L++)
		{
			tmpMyocardialPressureArray[L]=myocardialPressureScalingFactor*myocardialPressureArray[L];
		}

	bool isBreak = false;
	
//	output << "// minimum radius in structured tree r_min= " << 1000000*r_min << " micrometers " << endl;
//	output << "// mu_0 = " << mu_0 << " kg/(m*s) "<< endl;
	for(int cycleCounter=0; cycleCounter < maxNumberOfCycles; cycleCounter++)
	{

		for(int timeCounter=0;timeCounter < numberOfTimesteps; timeCounter++)
		{
		
	// check if periodicity to desired tolerance has occurred in subsequent cycles
		if(cycleCounter > 1)
		{
//		int integerMultiple = static_cast<int> (timeCounter*timeStepSize / cycleDuration);
//		float remainder = timeCounter * timeStepSize - static_cast<float> (integerMultiple) * cycleDuration;
//		output << "modulus of the doubles is: " << remainder << endl;

		if(timeCounter == numberOfTimesteps-1)//(remainder < 1e-09)
		{
	 		output << "timeCounter is " << timeCounter << ", the end of for-loop is reached. checking for periodicity." <<  endl; //remainder << endl;
			double deviation[numberOfTimesteps];
			double maxDeviation=0;
			int timestepOfMaxDeviation=0;
			for(int r=0; r<numberOfTimesteps; r++)
			{
//				double difference = 0;
				deviation[r] = fabs(capacitorVoltage[cycleCounter-2][r] - capacitorVoltage[cycleCounter-1][r]);
				output << " capacitorVoltage cycle " << cycleCounter-2 << " timeCounter " << r << ": " << capacitorVoltage[cycleCounter-2][r] << endl;
				output << " capacitorVoltage cycle " << cycleCounter-1 << " timeCounter " << r << ": " << capacitorVoltage[cycleCounter-1][r] << endl;

//				if(deviation[r] < 0)
//				{
//					output << "deviation was smaller 0" << endl;
//					deviation[r] = -1*difference;
					if(deviation[r] > maxDeviation)
					{
						maxDeviation = deviation[r];
						timestepOfMaxDeviation = r;
						output << " maxDeviation " << maxDeviation << " newly set at timestep " << r << endl;
					}
				output << " difference at cycle " << cycleCounter-1 << " timeCounter " << r << ": " << deviation[r] << endl;
				output << " maxDeviation at cycle " << cycleCounter-1 << " timeCounter " << timestepOfMaxDeviation << ": " << maxDeviation << endl;
//				}
//				else
//				{
//					output << "deviation was larger 0" << endl;
//					deviation[r] = difference;
//					if(deviation[r] > maxDeviation)
//					{
//						maxDeviation = deviation[r];
//						timestepOfMaxDeviation = r;
//						output << " maxDeviation " << maxDeviation << " newly set at timestep " << r << endl;
//					}
//				}

			}
			if(maxDeviation < tolerance)
			{
				output << "maxDeviation is smaller than desired tolerance after " << cycleCounter-1 << " cycles. Write resulting capacitor Pressures and capillaryPressure (BETWEEN ARTERIOLES AND CAPILLARIES!!) in file and leave! " << endl;
				for(int s=0; s<numberOfTimesteps; s++)
				{
				if(K==4)
				{
					output2 << s*timeStepSize << " " << capacitorVoltage[cycleCounter-1][s] << endl;
				}
//				finalOutput << s*timeStepSize << " " << myocardialPressureArray[s] - capacitorVoltage[cycleCounter-1][s] << endl;
				finalOutput << tmpMyocardialPressureArray[s] + capacitorVoltage[cycleCounter-1][s] + (R_capillary1/(R_arterial + R_capillary1) * (inletPressureArray[s] - (tmpMyocardialPressureArray[s] + capacitorVoltage[cycleCounter-1][s]))) << endl;
				finalOutput2 << capacitorVoltage[cycleCounter-1][s] << endl;
				}
				isBreak = true;
				cout << "termination criterion fulfilled after cycle: " << cycleCounter << ". " << endl;
			}
			else
			{
				output << " maxDeviation = " << maxDeviation << " is larger than desired tolerance " << tolerance << " at timestep " << timestepOfMaxDeviation << " within cycle, go on! " << endl;
			}

		}
		}

/*
		if(timeCounter==0)
		{
			capacitorVoltage[cycleCounter][timeCounter]=10;
//			newCapacitorVoltage[timeCounter%(2*numberOfTimesteps)]=10;
//			output << "capacitorVoltage values at time " << timeCounter*timeStepSize << " are: newCap " << newCapacitorVoltage[timeCounter%(2*numberOfTimesteps)] << "  and " << capacitorVoltage[timeCounter%(2*numberOfTimesteps)] << endl;
		}
*/
//		else
//		{
			output << "intraMyocardialPressureArray value at time " << timeCounter*timeStepSize << " is: " << tmpMyocardialPressureArray[timeCounter] << endl;





			capacitorVoltage[cycleCounter][timeCounter] = changingCapacitorVoltage(timeStepSize,  capacitorVoltage[cycleCounter][timeCounter-1], 
rho*(inletPressureArray[timeCounter] - (tmpMyocardialPressureArray[timeCounter] + capacitorVoltage[cycleCounter][timeCounter-1]))/(R_arterial + R_capillary1) + rho*(meanRightAtrialPressure - (tmpMyocardialPressureArray[timeCounter] + capacitorVoltage[cycleCounter][timeCounter-1]))/(R_capillary2 + R_venous),  
capacitorCurrent[timeCounter-1],
capacitance,
rho);

			modelCurrent[timeCounter] = rho*(inletPressureArray[timeCounter] - (tmpMyocardialPressureArray[timeCounter] + capacitorVoltage[cycleCounter][timeCounter-1]))/(R_arterial + R_capillary1);
			venousCurrent[timeCounter] = rho*(meanRightAtrialPressure - (tmpMyocardialPressureArray[timeCounter] + capacitorVoltage[cycleCounter][timeCounter-1]))/(R_capillary2 + R_venous);
			capacitorCurrent[timeCounter] = - modelCurrent[timeCounter] - venousCurrent[timeCounter];

			output << "modelCurrent: " << modelCurrent[timeCounter] << endl;
			output << "venousCurrent: " << venousCurrent[timeCounter] << endl;		
			output << "capacitorCurrent: " << capacitorCurrent[timeCounter] << endl;


/*			else
			{
			cout << " cycleCounter " << cycleCounter << " timeCounter " << timeCounter << "I do nothing" << endl;
			}
*/
//			output << "capacitorVoltage at time " << timeCounter*timeStepSize << " is: " << capacitorVoltage[timeCounter%(2*numberOfTimesteps)] << endl;
//		}
   	// store pressures in textfile:
//	cout << "resistance" << r << "    " << resistanceArray[r] << ";" << endl;
	output.precision(10);
	output << "capacitorVoltage at time " << timeCounter*timeStepSize << " in cycle " << cycleCounter << " is: " << capacitorVoltage[cycleCounter][timeCounter] << endl;


		}
	if (isBreak == true)
	{
		break;
	}
	}
//	numberedFilenames.clear();
	}
	output2.close();
	output.close();
	return 0;
}








