#include <iostream>
#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>
#include <fstream>
#include <sstream>
#include <stdint.h>
#include <string>
#include <vector>

using namespace std;

int getNumberOfLines(string file) // function to obtain number of outlets from file with listed outlet radii. Name of file is inserted in command line when programm is executed.
{
	int number_of_lines=0;
	string line;
	ifstream myfile(file.c_str());

	if(myfile.is_open())
	{
		while(!myfile.eof())
		{
			{
				getline(myfile,line);
				if (line.empty()) // make sure, only non-blank lines are counted!
				{
//					cout << "Empty line." << endl;
				}
				else
				{
//					cout << line << endl;
					number_of_lines++;
				}
			}
		}
		myfile.close();
	}

	cout << "Number of lines in " << file << " is " << number_of_lines << endl;
	
	return number_of_lines;
}

double getCenterOfInletPatch(int numberOfEntries, double centerpoint[])
{
	double result[3];
	double sumX=0, sumY=0, sumZ=0;
	string fileName;
	string line;
	fileName = "INLET.txt";
	ifstream infile(fileName.c_str());
	if(infile.fail())
	{
		cout << "Error: File INLET.txt could not be opened." << endl;
		return 1;
	}
	double arrayX[numberOfEntries];
	double arrayY[numberOfEntries];
	double arrayZ[numberOfEntries];

	for(int i=0; i<numberOfEntries; i++)
	{

//		getline(infile, line);
		infile >> arrayX[i];
		infile >> arrayY[i];
		infile >> arrayZ[i];

/*
		cout << "Inlet: arrayX[" << i << "]: " << arrayX[i] << endl;
		cout << "Inlet: arrayY[" << i << "]: " << arrayY[i] << endl;
		cout << "Inlet: arrayZ[" << i << "]: " << arrayZ[i] << endl;
*/
		sumX+= arrayX[i];
		sumY+= arrayY[i];
		sumZ+= arrayZ[i];		
	}
	infile.close();

	result[0]=sumX/numberOfEntries;
	result[1]=sumY/numberOfEntries;
	result[2]=sumZ/numberOfEntries;
/*
	cout << "x-coordinate of center of INLET.txt is: " << result[0] << endl;
	cout << "y-coordinate of center of INLET.txt is: " << result[1] << endl;
	cout << "z-coordinate of center of INLET.txt is: " << result[2] << endl;
*/
	for(int t=0; t<3; t++)
	{
		centerpoint[t]=result[t];
	}

	return 0;
}

double getCenterOfOutletPatch(string file, int numberOfEntries, double centerpoint[])
{
	double result[3];
	double sumX=0, sumY=0, sumZ=0;
	string line;
	ifstream infile(file.c_str());
	if(infile.fail())
	{
		cout << "Error: File " << file.c_str() << " could not be opened." << endl;
		return 1;
	}
	double arrayX[numberOfEntries];
	double arrayY[numberOfEntries];
	double arrayZ[numberOfEntries];
	stringstream fileName;
	for(int i=0; i<numberOfEntries; i++)
	{
//		getline(infile, line);
		infile >> arrayX[i];
		infile >> arrayY[i];
		infile >> arrayZ[i];

/*	
		cout << file.c_str() << " arrayX[" << i << "]: " << arrayX[i] << endl;
		cout << file.c_str() << " arrayY[" << i << "]: " << arrayY[i] << endl;
		cout << file.c_str() << " arrayZ[" << i << "]: " << arrayZ[i] << endl;
*/
		sumX+= arrayX[i];
		sumY+= arrayY[i];
		sumZ+= arrayZ[i];	

	}
	infile.close();

	result[0]=sumX/numberOfEntries;
	result[1]=sumY/numberOfEntries;
	result[2]=sumZ/numberOfEntries;
/*
	cout << "x-coordinate of center of " << file.c_str() <<  " is: " << result[0] << endl;
	cout << "y-coordinate of center of " << file.c_str() <<  " is: " << result[1] << endl;
	cout << "z-coordinate of center of " << file.c_str() <<  " is: " << result[2] << endl;
*/
	for(int t=0; t<3; t++)
	{
		centerpoint[t]=result[t];
	}

	return 0;
}


int main()
{
	int N_outlets = 0;
	double scalingFactor = 0.001;

	cout << "What is the number of outletPatches?" << endl;
	cin >> N_outlets;

	cout << "What is the scalingFactor of the model?" << endl;
	cin >> scalingFactor;

	double inletCenterpoint[3];
	double outletCenterpoint[N_outlets][3];

//	int numberOfVerticesInlet=0;
	int numberOfVertices[N_outlets];

	stringstream tmpFileName;
	string fileName;

//	numberOfVerticesInlet = getNumberOfLines("INLET.txt");
//	getCenterOfInletPatch(numberOfVerticesInlet, inletCenterpoint);

	for(int j=0; j<N_outlets; j++)
	{
		double tmpOutletCenterpoint[3];
		tmpFileName << "OUTLET" << j+1 << ".txt";
		fileName = tmpFileName.str();
		numberOfVertices[j+1] = getNumberOfLines(fileName);
		getCenterOfOutletPatch(fileName, numberOfVertices[j+1], tmpOutletCenterpoint);
		tmpFileName.str("");
		for(int z=0; z<3; z++)
		{
			outletCenterpoint[j][z]=tmpOutletCenterpoint[z];
		}
	}


//	cout << "inletCenterpoint coordinates are: " << inletCenterpoint[0] << ", " << inletCenterpoint[1] << ", " << inletCenterpoint[2] << ". " << endl;
	for(int j=0; j<N_outlets; j++)
	{
		cout << "outletCenterpoint coordinates for outlet " << j+1 << " are: " << outletCenterpoint[j][0] << ", " << outletCenterpoint[j][1] << ", " << outletCenterpoint[j][2] << ". " << endl;
	}

	double xSum=0, ySum=0, zSum=0;
	double baryCenterCoordinate[3];
	for(int n=0; n<N_outlets; n++)
	{
		xSum += outletCenterpoint[n][0];
		ySum += outletCenterpoint[n][1];
		zSum += outletCenterpoint[n][2];
	}
	baryCenterCoordinate[0]=xSum/N_outlets;
	baryCenterCoordinate[1]=ySum/N_outlets;
	baryCenterCoordinate[2]=zSum/N_outlets;

	cout << "baryCenter is at: " << baryCenterCoordinate[0] << ", " << baryCenterCoordinate[1] << ", " << baryCenterCoordinate[2] << ", or scaled: " << baryCenterCoordinate[0]*scalingFactor << ", " << baryCenterCoordinate[1]*scalingFactor << ", " << baryCenterCoordinate[2]*scalingFactor << endl;

	double distances[N_outlets];

	for(int n=0; n<N_outlets; n++)
	{
		double squareSum=0;
		double result=0; 
		for(int m=0; m<3; m++)	
		{
			squareSum += pow(baryCenterCoordinate[m]-outletCenterpoint[n][m],2);
		}
		result = sqrt(squareSum)*scalingFactor;
		distances[n] = result;
		cout << "Distance baryCenter to outlet" << n+1 << " is: " << 1000*distances[n] << " mm. " << endl;
	}

	double maxDistance=0, minDistance=1000;
	for(int n=0; n<N_outlets; n++)
	{
		if(maxDistance < distances[n])
		{
			maxDistance = distances[n];
		}
		if(minDistance > distances[n])
		{
			minDistance = distances[n];
		}
		cout << "maxDistance: " << maxDistance << " m , minDistance: " << minDistance << " m." << endl;
	}	
	
	double difference=0;
	difference = maxDistance - minDistance;
	cout << "difference: " << difference << " m." << endl;

	ofstream output("ventricularPressureScaling.txt");

	double scalingFactors[N_outlets];
	for(int n=0; n<N_outlets; n++)
	{
		scalingFactors[n] = (distances[n]-minDistance)/difference;
		cout << "scalingFactor @outlet" << n+1 << ": " << scalingFactors[n] << endl;
		output.precision(10);
		output << scalingFactors[n] << endl;
	}

	return 0;
}








