#include <iostream>
#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>
#include <fstream>
#include <sstream>
#include <stdint.h>
#include <string>

using namespace std;

int getNumberOfOutlets(string file) // function to obtain number of outlets from file with listed outlet radii. Name of file is inserted in command line when programm is executed.
{
	int number_of_lines=0;
	string line;
	ifstream myfile(file.c_str());

	if(myfile.is_open())
	{
		while(!myfile.eof())
		{
			{
				getline(myfile,line);
				if (line.empty()) // make sure, only non-blank lines are counted!
				{
//					cout << "Empty line." << endl;
				}
				else
				{
//					cout << line << endl;
					number_of_lines++;
				}
			}
		}
		myfile.close();
	}

//	cout << "Number of lines in " << file << " is " << number_of_lines << endl;
	
	return number_of_lines;
}

double readFile(string fileName, double array[])
{
	ifstream infile(fileName.c_str());
	int s=1;
	string line;
	stringstream Str;

	if(infile.is_open())
	{
		while(!infile.eof())
		{
			{
				getline(infile,line);
				if (line.empty()) //make sure empty lines are not read into array.
				{
//					cout << "Empty line." << endl;
				}
				else
				{
					Str << line;
					Str >> array[s];
					Str.clear(); // Stringstream needs to be cleared in order to be written correctly in next step.
//					cout << "outletRadius" << s << ":" << outletRadiusArray[s] << endl;
					s++;
				}
			}
		}
		infile.close();
	}

	return 0;
}

double MU_app_rel(double diameter) // apparent relative blood viscosity after Pris et al. diameter in micrometers
{
	double result = 0;
		result = 220 * exp(-1.3*diameter) + 3.2 - 2.44 * exp(-0.06*pow(diameter , 0.645));
//		cout << "mu_rel = " << result << endl;
	return result;
}

int main()
{
	int N_outlets = 0;
	string filename1;
	string filename2;
	string filename3;
	string filename4;
	string filename5;
	string filename6;
	string filename7;
	string filename8;

// choose input file with list of outlet radii (in mm)
	cout << "What is the name of the input-File with outlet-radii in millimeters?" << endl;
	cin >> filename1;
	
	N_outlets = getNumberOfOutlets(filename1);
	cout << "Number of outlets is: " << N_outlets << endl;

// arrays for resistances and outletradii are created
	double resistanceFractionArray[N_outlets];
	double finalVesselResistanceArray[N_outlets];
	double capillaryResistanceArray[N_outlets];
	double venousResistanceArray[N_outlets];
	double outletRadiusArray[N_outlets];

// readin of outletradii from file into outletRadiusArray
	readFile(filename1, outletRadiusArray);

	for(int i=1; i<=N_outlets; ++i)
	{
		cout << "OutletRadius" << i << " is " << outletRadiusArray[i] << endl;
	}

	int N_orders=11;
	double lengthArray[N_orders];
	double SDlengthArray[N_orders];
	double maxDiameterArray[N_orders];
	double meanDiameterArray[N_orders];
	double volumeArray[N_outlets];
	double numberOfVesselsArray[N_orders];
	double distanceArray[N_outlets];
	double inletDiameter = 0;
	double diameterScaleRatio = 0;
	double lengthScaleRatio[N_outlets];
	double tmpLengthScaleRatio[N_outlets];
	double minLengthScaleRatio=100, maxLengthScaleRatio=0;
	double SDlengthScaleRatio[N_outlets];
	double distanceScaleRatio[N_outlets];
	double maxDistance=0;
	double minDistance=100;
	double treeScaleRatio = 0;
	double maxOutletDiameter=0;
	double minOutletDiameter=100;
	double maxNumber[N_outlets];
	double numberScaleRatio[N_outlets];
	double sumLength[N_outlets];
	double meanNumber=0;
	double totalResistance=0;
	double equivalentResistance[N_outlets];
	double modelVolume=0;
	filename2 = "lengths.txt";
	filename3= "maxDiameters.txt";
	filename4 = "meanDiameters.txt";
	filename8 = "volumes.txt";
	filename5 = "numbers.txt";
	filename6 = "distances.txt";

	cout << "name of the input-File with lengths of vessels in millimeters: " << filename2 << endl;
	cout << "name of the input-File with max-diameter in micrometers: " << filename3 << endl;
	cout << "name of the input-File with mean-diameters in micrometers: " << filename4 << endl;
	cout << "name of the input-File with volumes between inlet and outlets in m^3:" << filename8 << endl;
	cout << "name of the input-File with number of vessels in orders: " << filename5 << endl;
	cout << "name of input-File with distances between inlet and outlets in millimeters: " << filename6 << endl;


	cout << "Please insert inlet diameter of the model in micrometers" << endl;
	cin >> inletDiameter;

	readFile(filename2, lengthArray);
	readFile(filename3, maxDiameterArray);
	readFile(filename4, meanDiameterArray);
	readFile(filename8, volumeArray);
	readFile(filename5, numberOfVesselsArray);
	readFile(filename6, distanceArray);
	diameterScaleRatio = inletDiameter/maxDiameterArray[N_orders];
	cout << "inletDiameter: " << inletDiameter/1000 << " mm." << endl;
	cout << "diameterScaleRatio: " << diameterScaleRatio << endl;

	for(int i=1; i<=N_orders; ++i)
	{
		cout << "length in order" << i << " is " << lengthArray[i] << endl;
	}

	for(int i=1; i<=N_orders; ++i)
	{
		cout << "maxDiameter in order" << i << " is " << maxDiameterArray[i] << endl;
	}

	for(int i=1; i<=N_orders; ++i)
	{
		cout << "meanDiameter in order" << i << " is " << meanDiameterArray[i] << endl;
	}

	double sumModelVolumes=0;
	for(int i=1; i<=N_outlets; ++i)
	{
		cout << "volume between inlet and outlet" << i << " is " << volumeArray[i] << endl;
		sumModelVolumes += volumeArray[i];
	}
		cout << "sumModelVolumes (sum of volumes between inlet and outlets): " << sumModelVolumes << endl;

	for(int i=1; i<=N_orders; ++i)
	{
		cout << "number in order" << i << " is " << numberOfVesselsArray[i] << endl;
	}

	for(int i=1; i<=N_outlets; ++i)
	{
		cout << "distance to outlet " << i << " is " << distanceArray[i] << endl;
	}

	double stressRestScaleFactor = 1;
	cout << "Please insert vasodilation factor of vessels behind model outlet (usual range: 1 or 1.15):" << endl;
	cin >> stressRestScaleFactor;

	cout << "stressRestScaleFactor: " << stressRestScaleFactor << endl;

	double dilatationScaleArray[N_orders];
	if(stressRestScaleFactor==1.15)
	{
	dilatationScaleArray[1]=1.4;
	dilatationScaleArray[2]=1.37;
	dilatationScaleArray[3]=1.33;
	dilatationScaleArray[4]=1.28;
	dilatationScaleArray[5]=1.2;
	dilatationScaleArray[6]=1.15;
	dilatationScaleArray[7]=1;//1.15;
	dilatationScaleArray[8]=1;//1.15;	
	dilatationScaleArray[9]=1;//1.15;
	dilatationScaleArray[10]=1;//1.15;
	dilatationScaleArray[11]=1;//1.15;
	}
	else if(stressRestScaleFactor==1)
	{
	dilatationScaleArray[1]=1;
	dilatationScaleArray[2]=1;
	dilatationScaleArray[3]=1;
	dilatationScaleArray[4]=1;
	dilatationScaleArray[5]=1;
	dilatationScaleArray[6]=1;
	dilatationScaleArray[7]=1;
	dilatationScaleArray[8]=1;	
	dilatationScaleArray[9]=1;
	dilatationScaleArray[10]=1;
	dilatationScaleArray[11]=1;
	}	

	for(int i=1; i<=N_orders; ++i)
	{
		cout << "dilatationScaleFactor in order " << i << " is " << dilatationScaleArray[i] << endl;
	}	


	for(int n=1; n<=N_outlets; ++n) 
	{
		numberScaleRatio[n]=0;
		sumLength[n]=0;
		for(int m=1; m<=N_orders; ++m) 
		{

			if(2*outletRadiusArray[n]*1000 > diameterScaleRatio*maxDiameterArray[m]) //assign maxNumber of vessels in generation corresponding to outletSize for later computation of outletNumberRatio
			{
				sumLength[n] += numberOfVesselsArray[m]*lengthArray[m];
				maxNumber[n] = numberOfVesselsArray[m];
//				cout << "maxNumber for outlet" << n << " assigned @ order " << m << ": " << maxNumber[n] << endl;
			}

			if(2*outletRadiusArray[n]*1000 < diameterScaleRatio*maxDiameterArray[m]) //assign sum of lenghts of vessels before micro-vessels to sumLength for later computation of ratio distance to sumLength
			{
				sumLength[n] += lengthArray[m];
//				cout << "sumLength for outlet" << n << " assigned @ order " << m << ": " << sumLength[n] << endl;
//				maxNumbers = numberOfVesselsArray[m];
			}

		}

	
		numberScaleRatio[n] = N_outlets/maxNumber[n];
		cout << "maxNumber for outlet" << n << ": " << maxNumber[n] << endl;
		cout << "numberScaleRatio at outlet" << n << " is " << ": " << numberScaleRatio[n] << endl;
		cout << "sumLength for outlet" << n << " is: " << sumLength[n] << endl;

	}
	
	double sumNumbers=0;
	double sumDistances=0;
	double meanDistance=0;
	for(int n=1; n<=N_outlets; ++n) 
	{
		sumDistances=distanceArray[n];
		sumNumbers+=maxNumber[n];	
	}
	meanDistance=sumDistances/N_outlets;
	meanNumber=sumNumbers/N_outlets;
	cout << "mean number is: " << meanNumber << endl;
	cout << "mean distance is: " << meanDistance << endl;
	

	double testRadius=0;
	double mu_0=0.00488, rho=1060; //[kg/(mm*s)]
	cout << "Standard viscosity is: " << mu_0 << " kg/(m*s) " << endl;
	cout << "density is: " << rho << " kg/m^3 " << endl;




	double sumDistance[N_outlets];
	double distanceScaling[N_outlets];
	double scaledNumberOfVessels[N_outlets];
	double numberOfVesselsScaling[N_outlets];
	for(int r=1; r<=N_outlets; r++)
	{
	double oneVesselDistance = 0;
	sumDistance[r] =0;

	for(int i=1; i<=N_orders; ++i) 
	{
		if(2*outletRadiusArray[r]*1000 > diameterScaleRatio*meanDiameterArray[i-1] && 2*outletRadiusArray[r]*1000 < diameterScaleRatio*meanDiameterArray[i])

		{
			cout << "outletDiameter" << r << " is smaller than maxDiameter at order " << i << " but larger than maxDiameter at order " << i-1 << endl;	
			cout << diameterScaleRatio*meanDiameterArray[i-1] << " < " << 	2*outletRadiusArray[r]*1000 << " < " << diameterScaleRatio*meanDiameterArray[i] << endl;
			cout << "numberOfVesselsArray[i-1]: " << numberOfVesselsArray[i-1] << endl;
			cout << "numberOfVesselsArray[i] - numberOfVesselsArray[i-1]: " << numberOfVesselsArray[i] - numberOfVesselsArray[i-1] << endl;
			cout << "meanDiameterArray[i] - meanDiameterArray[i-1]: " << diameterScaleRatio*(meanDiameterArray[i] - meanDiameterArray[i-1]) << endl;
			cout << "(2*outletRadiusArray[r]*1000) - meanDiameterArray[i-1]: " << (2*outletRadiusArray[r]*1000) - diameterScaleRatio*meanDiameterArray[i-1] << endl;
			numberOfVesselsScaling[r] = numberOfVesselsArray[i-1] + (numberOfVesselsArray[i] - numberOfVesselsArray[i-1])/(diameterScaleRatio*meanDiameterArray[i] - diameterScaleRatio*meanDiameterArray[i-1])*((2*outletRadiusArray[r]*1000) - diameterScaleRatio*meanDiameterArray[i-1]);
		}
				
	}
		distanceScaling[r] = sumDistance[r]/oneVesselDistance;
		cout << "sumDistance at outlet" << r << ": " << sumDistance[r] << endl;	
		cout << "distanceScaling at outlet" << r << ": " << distanceScaling[r] << endl;
		cout << "oneVesselDistance at outlet" << r << ": " << oneVesselDistance << endl;
		cout << "numberOfVesselsScaling at outlet" << r << ": " << numberOfVesselsScaling[r] << endl;
	}

	double meanDistanceScaling = 0;
	double meanNumberOfVesselsScaling = 0;
	for(int r=1; r<=N_outlets; r++)
	{
		meanNumberOfVesselsScaling += scaledNumberOfVessels[r];
		meanDistanceScaling += distanceScaling[r];
	}
	meanNumberOfVesselsScaling = meanNumberOfVesselsScaling/N_outlets;
	meanDistanceScaling = meanDistanceScaling/N_outlets;
	cout << "meanDistanceScaling is: " << meanDistanceScaling << endl;	
	cout << "meanNumberOfVesselsScaling is: " << meanNumberOfVesselsScaling << endl;
	cout << " meanDistance is : " << meanDistance << endl;

	double equivalentVolume[N_outlets];
	for(int i=1; i<=N_outlets; i++)
	{
		equivalentVolume[i] = N_outlets*volumeArray[i];
		cout << "equivalent Volume @outlet " << i << " is: " << equivalentVolume[i] << endl;
	}				

	double sumVolumes[N_outlets];
	double volumeScaling[N_outlets];
	for(int r=1; r<=N_outlets; r++)
	{
	double oneVesselVolume = 0;
	sumVolumes[r] =0;
	for(int i=1; i<=N_orders; ++i) 
	{
		if(2*outletRadiusArray[r]*1000 < diameterScaleRatio*maxDiameterArray[i])
		{
			cout << "outletDiameter" << r << " is smaller than maxDiameter at order " << i << endl;	

				sumVolumes[r] += M_PI*meanDiameterArray[i]/2000000*0.001*lengthArray[i]*numberOfVesselsArray[i]*numberScaleRatio[i];//
				cout << "orderCounter is " << i << endl;
				cout << "sumVolumes at outlet" << r << ": " << sumVolumes[r] << endl;		
//			}
			oneVesselVolume += M_PI*meanDiameterArray[i]/2000000*0.001*lengthArray[i];		
		}
	}
		volumeScaling[r] = sumVolumes[r]/oneVesselVolume;
		cout << "sumVolume at outlet" << r << ": " << sumVolumes[r] << endl;	
		cout << "volumeScaling at outlet" << r << ": " << volumeScaling[r] << endl;
		cout << "oneVesselVolume at outlet" << r << ": " << oneVesselVolume << endl;
	}

	double meanVolume = 0;
	double meanVolumeScaling = 0;
	for(int r=1; r<=N_outlets; r++)
	{
		meanVolumeScaling += volumeScaling[r];
		meanVolume += volumeArray[r];
	}
	meanVolumeScaling = meanVolumeScaling/N_outlets;
	cout << "meanVolumeScaling is: " << meanVolumeScaling << endl;	
	meanVolume = meanVolume/N_outlets;
	cout << "meanVolume between inlet and outlets is: " << meanVolume << " m^3." << endl;
	


	ofstream output3("modelResistances.txt");


	for(int i=1; i<=N_outlets; i++)
	{
//		cout << " scaledNumberOfVessels correct? outlet" << i << ": " << numberOfVesselsScaling[i] << endl;
		double result=0;
		equivalentResistance[i]=0;
		result=8*M_PI*mu_0*MU_app_rel((inletDiameter+2*1000*outletRadiusArray[i])/2)/3.2 
		* /*numberOfVesselsScaling[i] */pow(0.001*distanceArray[i],3)//pow(0.001*meanDistance,3) //
		/ (pow(/*meanVolume*/volumeArray[i],2));//(pow(modelVolume,2));//
		equivalentResistance[i]=result;
	cout << "vessel Resistance @ outlet" << i << " is: " << equivalentResistance[i] << endl;
	output3.precision(10);
	output3 << equivalentResistance[i] << endl;
	}

	string startOff;
	cout << "Now it goes off!" << endl;
	cin >> startOff;

//	string outputfilename;
//	cout << "What is the desired outputfilename?" << endl;
//	cin >> outputfilename;
//	ofstream output(outputfilename.c_str());
	ofstream output("finalVesselResistances.txt");
	ofstream output4("venousResistances.txt");



	double fullTreeVolume = 0;
	double fullTreeCumulativeLength = 0;
	double scalingLawConstant = 0;
	double resistanceScalingConstant = 0;
	double maxResistance = 0;
	double epsilon = -0.6;
	cout << "exponent of reference Volume, epsilon = " << epsilon << endl;
	
	for(int i=1; i<=N_orders; ++i) 
	{
		fullTreeVolume += numberOfVesselsArray[i]*M_PI*pow(diameterScaleRatio*dilatationScaleArray[i]*0.000001*meanDiameterArray[i]/2,2)*0.001*lengthArray[i];//*diameterScaleRatio;
		fullTreeCumulativeLength += numberOfVesselsArray[i]*0.001*lengthArray[i];//*diameterScaleRatio;
	}

	scalingLawConstant = fullTreeVolume/(pow(inletDiameter/1000000, 0.667) * fullTreeCumulativeLength);


	cout << "Cumulative length of total vascular tree (all orders 1-11) is: fullTreeCumulativeLength " << fullTreeCumulativeLength << " m" << endl;
	cout << "scalingLawConstant = fullTreeVolume/(inletDiameter^2/3 * fullTreeCumulativeLength) : " << scalingLawConstant << " m^(4/3)" << endl;
	cout << "Volume of total vascular tree (all orders 1-11) is: fullTreeVolume " << fullTreeVolume << " m^3" << endl;
		
	cout << "sumModelVolumes: " << sumModelVolumes << " m^3" << endl;


	double crownVolumes[N_outlets], crownVolumes_2[N_outlets], cumulativeCrownLengths[N_outlets], cumulativeCrownLengths_2[N_outlets];

	for(int t=1; t<=N_outlets; ++t)
	{
	cumulativeCrownLengths[t] = 0;
	cumulativeCrownLengths_2[t] = fullTreeCumulativeLength * pow(2*0.001*outletRadiusArray[t]/(0.000001*inletDiameter),2.333);
	for(int l=1; l<=N_orders; ++l)
	{
		if(2*outletRadiusArray[t]*1000 > diameterScaleRatio*maxDiameterArray[l])
		{
			cumulativeCrownLengths[t] += numberOfVesselsArray[l]/numberOfVesselsScaling[t] *0.001*lengthArray[l]*diameterScaleRatio;
		}
	}
	}

	for(int t=1; t<=N_outlets; ++t)
	{
		crownVolumes_2[t] = fullTreeVolume * pow(2*0.001*outletRadiusArray[t]/(0.000001*inletDiameter),3);			
		
		crownVolumes[t] = scalingLawConstant * pow(2*outletRadiusArray[t]/1000,0.667) *cumulativeCrownLengths_2[t];
		cout << "crownVolume @ outlet" << t << ": " << crownVolumes[t] << " m^3." << endl;
		cout << "crownVolume_2 @ outlet" << t << ": " << crownVolumes_2[t] << " m^3." << endl;
		cout << "cumulativeCrownLength @ outlet" << t << ": " << cumulativeCrownLengths[t] << " m." << endl;
		cout << "cumulativeCrownLength_2 @ outlet" << t << ": " << cumulativeCrownLengths_2[t] << " m." << endl;
	}

	double meanInletPressure = 0;
	double meanRightAtrialPressure = 0.47, meanArterialPressureLoss = 0, meanModelVolumeFlow = 0, meanCapillaryPressure = 0;
	cout << "Is this human or pig data? Enter 0 for pig, 1 for human. " << endl;
	bool humanOrPig;
	double stemFlowCrownLengthScaling = 0;	
	cin >> humanOrPig;
	if(humanOrPig == 0)
	{	
		stemFlowCrownLengthScaling = 6E-9/pow(diameterScaleRatio,2.333); //Factor 6E-9 m^3/(s*m) is stemFlowCrownLengthScaling for pigs from Kassab et al, PhysMedBiol 1999
	}
	else //humanOrPig == 1
	{
		stemFlowCrownLengthScaling = 6E-9 * pow(diameterScaleRatio,2.333); //Factor 6E-9 m^3/(s*m) is stemFlowCrownLengthScaling for pigs from Kassab et al, PhysMedBiol 1999
	}

	cout << "What is mean inlet Pressure @rest in m^2/s^2?" << endl;
	cin >> meanInletPressure; 	
	cout << "Enter mean capillary pressure in m^2/s^2:" << endl;
	cin >> meanCapillaryPressure;
//	cout << "Enter mean model volume flow in m^3/s: " << endl;
//	cin >> meanModelVolumeFlow;

	cout << "meanInletPressure: " << meanInletPressure << " m^2/s^2." << endl;	
//	cout << "meanRightAtrialPressure: " << meanRightAtrialPressure << " m^2/s^2." << endl;	
	cout << "meanCapillaryPressure: " << meanCapillaryPressure << " m^2/s^2." << endl;		
//	cout << "meanModelVolumeFlow: " << meanModelVolumeFlow << " m^3/s." << endl;	
	cout << "stemFlowCrownLengthScaliing: " << stemFlowCrownLengthScaling << " m^2/s." << endl;		

	meanModelVolumeFlow = stemFlowCrownLengthScaling * fullTreeCumulativeLength;


	if(stressRestScaleFactor==1)
	{
//	meanArterialPressureLoss = (meanInletPressure - meanRightAtrialPressure) * 0.6; //fraction of pressure loss across arteries/arterioles in coronary vasculature at rest
	meanArterialPressureLoss = meanInletPressure - meanCapillaryPressure; //fraction of pressure loss across arteries/arterioles in coronary vasculature. stress or rest state already accounted for in meanCapillaryPressure
	}
	else if(stressRestScaleFactor==1.15)
	{
//	meanArterialPressureLoss = (meanInletPressure - meanRightAtrialPressure) * 0.25;//fraction of pressure loss across arteries/arterioles in coronary vasculature at stress
	meanArterialPressureLoss = meanInletPressure - meanCapillaryPressure; //fraction of pressure loss across arteries/arterioles in coronary vasculature. stress or rest state already accounted for in meanCapillaryPressure
	meanModelVolumeFlow *= 4; //increased blood flow at stress
	}	
	cout << "meanArterialPressureLoss: " << meanArterialPressureLoss << endl;
	cout << "meanModelVolumeFlow: " << meanModelVolumeFlow << " m^3/s." << endl;	
	maxResistance = rho * meanArterialPressureLoss / meanModelVolumeFlow;	



/*
	cout << "enter maxResistance of full coronary tree (inlet to precapillary level) in kg/(m^4*s): " << endl;
	cin >> maxResistance;	
*/
	cout << "maxResistance: " << maxResistance << " kg/(m^4*s)" << endl;
	resistanceScalingConstant = maxResistance * pow(inletDiameter/1000000,4) / fullTreeCumulativeLength;
	cout << "resistanceScalingConstant: R_max*D_max^4/L_max: " << resistanceScalingConstant << " kg/(m*s)" << endl;	
	

	double crownResistanceArray[N_outlets];

	for(int r=1; r<=N_outlets; ++r)
	{

	crownResistanceArray[r] = resistanceScalingConstant * cumulativeCrownLengths_2[r]/(pow(2*0.001*outletRadiusArray[r],4));


	finalVesselResistanceArray[r] = crownResistanceArray[r];
	cout << "crownResistance @ outlet" << r << ": " << finalVesselResistanceArray[r] << endl;	;	
	if(stressRestScaleFactor==1)
	{
	capillaryResistanceArray[r] = finalVesselResistanceArray[r]/5;
	}
	else if(stressRestScaleFactor==1.15)
	{
	capillaryResistanceArray[r] = finalVesselResistanceArray[r]*1.44;
	}	
	cout << "capillaryResistance @ outlet" << r << ": " << capillaryResistanceArray[r] << endl;
	finalVesselResistanceArray[r] += capillaryResistanceArray[r] + equivalentResistance[r];
	cout << "finalVesselResistance after addition of capillary contribution and equivalentResistance before outlet is: " << finalVesselResistanceArray[r] << endl;	
	if(stressRestScaleFactor==1)
	{
	venousResistanceArray[r] = capillaryResistanceArray[r] + 1.2*capillaryResistanceArray[r];
	}
	else if(stressRestScaleFactor==1.15)
	{
	venousResistanceArray[r] = capillaryResistanceArray[r] + 0.03*capillaryResistanceArray[r];
	}	
	cout << "capillary + venous Resistance @ outlet" << r << ": " << venousResistanceArray[r] << endl;		
	
// store resistances in textfile:
	output.precision(10);
	output4.precision(10);
	output << finalVesselResistanceArray[r] << endl;
	output4 << venousResistanceArray[r] << endl;

	}

	return 0;
}








